try:
    import ConfigParser as configparser # P2
except ImportError:
    import configparser # P3

from collections import defaultdict

"""
    INI file parser for our custom configurations/constants
    Notes to check before modifying the config file!!
        - Take a look at the file format =>
          https://en.wikipedia.org/wiki/INI_file
        - Sections and variable keys are case sensitive.
    The values inside the "DEFAULT" section are present in every other sections
    (it is like the base values) and they are overwritten by the values set in
    each sections, e.g.:
        [DEFAULT]
        host = 127.0.0.1
        port = 8069
        version = master
        [11.0]
        version = 11.0
        [saas-11.3]
        version = saas-11.3
        host = 78.200.0.0
        extra = VF
    Values stored in the dict:
        {
            '11.0': {
                'host': '127.0.0.1',
                'port': '8069',
                'version': '11.0'
            },
            'saas-11.3': {
                'host': '78.200.0.0',
                'port': '8069',
                'version': 'saas-11.3',
                'extra': 'VF',
            }
        }
    The config is stored in the `saas_config` dict. We have put the config
    inside a dict to avoid specific behavior from the config parser that tries
    to behave as close as a dict but do not really. For example `get` doesn't
    behave like a dict which can be confusing.
    By default the values are considered as strings. If you wish to specify a
    type on the values you can use those prefixes on the variable name `f_` for
    floats `i_` for integers `s_` for strings `b_` for booleans, the value
    should be any of "y yes 1 true t on" for True or "n no 0 false f off" for
    False, anything else will be considered as False.
"""


CONFIG_FILE_PATH = '/etc/odoo/odoo.conf'


def read_saas_config(path=CONFIG_FILE_PATH):
    # we are using the RawConfigParser to avoid issues with python 2/3 when trying to disable the interpolation
    config_parser = configparser.RawConfigParser()
    # to make the sections & keys case sensitive
    config_parser.optionxform = str
    # doesn't care if the file doesn't exist
    config_parser.read(path)

    types = {
        'f_': float,
        'b_': lambda x: x.lower() in 'y yes 1 true t on'.split(),
        's_': str,
        'i_': int,
    }
    # retrieve the items from the default section to append them to the others sections
    default_items = list(config_parser.defaults().items())
    # read through all the sections (except DEFAULT) and store the values in the dict
    res = defaultdict(dict)
    for s in config_parser.sections():
        for k, v in default_items + config_parser.items(s):
            key = k
            converter = str
            if types.get(k[:2]):
                key = k[2:]
                converter = types[k[:2]]
            res[s][key] = converter(v)
    return res


saas_config = read_saas_config()
